#ifndef CELL_H_
#define CELL_H_

// Contains Spreadsheet and Cell classes.

#include <iosfwd>
#include <vector>
#include "Defs.h"
#include "PubSub.h"
#include "Function.h"

// A Cell is a container of a value.  It has
// a location within a Spreadsheet.  It is
// responsible for updating itself in response
// to changes in other Cells.
class Cell : public Publisher, public ISubscriber
{
public:
    Cell(ICellContainer *cnt=NULL);
    Cell(Cell const &);
    virtual ~Cell();

    Cell &operator=(Cell const &);

    // Implementation of ISubscriber:
    virtual void update(Publisher const *p);

    // Cell operations:

    // Sets container of cell:
    void setContainer(ICellContainer *cnt);

    // Retrieve the most current value of the cell:
    Value getValue() const {return cachedValue;}

    // Set the cell's value (removes any current function)
    void setValue(Value const &v);

    // Set the cell's function (removes any current value)
    // (Note: The Cell takes ownership of the function object.)
    void setFunc(std::string const &funcText);

    // Display the cell's contents
    void printOn(std::ostream &, bool showFuncs=false) const;

    // Clear the cell to a null state
    void clear();
    
    // Tell if the cell is in a null state
    bool isNull() const;    
private:
    static const Value NULL_VALUE; // Used to indicate null state
    
    Value cachedValue;      // Cached value of the cell
    Formula formula;        // Associated formula
    ICellContainer *container; // Container of this cell

    mutable bool inUpdate;  // Used to detect update loops
};

#endif
