#ifndef DEFS_H_
#define DEFS_H_

#include <exception>

// The type of data held in Cell
// objects and calculated by
// Function objects:
typedef double Value;


// A utility class to automatically set a
// variable to its default value.  For
// scalar (integer, boolean) types, the
// default value is 0 or false.
template <class T>
class auto_resetter
{
public:
   auto_resetter(T &v) : vr(v) {}
   ~auto_resetter() {vr = T();}
   T &vr;
};

// A generic exception for spreadsheet classes.
class SpreadsheetError : public std::exception
{
public:
    SpreadsheetError(std::string const &m) : msg(m) {}
    const char *what() {return msg.c_str();}
private:
    std::string msg;
};

#endif