#ifndef ICELLCONTAINER_H
#define ICELLCONTAINER_H

#include "Defs.h"

class Cell;

// ICellContainer is an abstract interface; its main
// purpose is to help decouple Formula and Spreadsheet
// a bit.
class ICellContainer
{
public:
    // Get access to a cell based on an arbitrary string
    // address (adds cell if necessary):
    virtual Cell &at(std::string const &address) = 0;

    // Look up cell by address; returns NULL if cell does
    // not currently exist:
    virtual Cell const *findCell(std::string const &address) const = 0;

    // Look up cell by address; returns NULL if cell does
    // not currently exist:
    virtual Cell *findCell(std::string const &address) = 0;

    // Exceptions thrown by at() functions:
    class BadCellRef : public SpreadsheetError
    {
    public:
        BadCellRef(std::string const &s) : SpreadsheetError(s) {}
    };
    
};


#endif