#ifndef PUBSUB_H_
#define PUBSUB_H_

// Contains Publisher and ISubscriber classes.

#include <list>

class ISubscriber;

// Implementation of the Publisher/Subscriber
// pattern [GOF]

// A Publisher keeps track of its subscribers
// and can notify them of interesting events.
class Publisher
{
public:
    void notify() const;
    void subscribe(ISubscriber *sub);
    void unsubscribe(ISubscriber *sub);
private:
    typedef std::list<ISubscriber*> SubColl;
    SubColl subColl;
};


// ISubscriber provides an interface for
// subscribers to be notified about data
// changes.
class ISubscriber
{
public:
    virtual void update(const Publisher *pub) = 0;
    virtual ~ISubscriber();
};

#endif
