#ifndef SPREADSHEET_H_
#define SPREADSHEET_H_

#include <iosfwd>
#include <vector>
#include <memory>
#include "Defs.h"
#include "PubSub.h"
#include "ICellContainer.h"
#include "Cell.h"


// A Spreadsheet is a container of Cells.  It is responsible
// for creating and destroying Cells and displaying them
// in a 2D array.
class Spreadsheet : public ICellContainer
{
public:
    Spreadsheet();
    Spreadsheet(Spreadsheet const &other);
    ~Spreadsheet();

    Spreadsheet &operator=(Spreadsheet const &other);

    // Set the size of the spreadsheet (destroys current
    // values):
    void setSize(int rows,int cols);

    // Ensure that the spreadsheet is big enough to
    // contain row, col (1-based):
    void ensureSize(int row, int col);

    // Get access to a cell at (row,col); row, col are 
    // 1-based:
    Cell &at_rowcol(int row, int col);
    Cell const &at_rowcol(int row, int col) const;

    // Set a cell at a particular address:
    //void setCell(std::string const &address, std::auto_ptr<Cell> & cellPtr);

    // Get access to a cell based on an "A1" style reference
    virtual Cell &at(std::string const &address);

    // Look up cell, returning NULL if it does not exist:
    virtual Cell const *findCell(std::string const &address) const;
    virtual Cell *findCell(std::string const &address);

    // Print the spreadsheet:
    void printOn(std::ostream &, bool showFuncs = false) const;

    // Clear the spreadsheet's functions and values:
    void clear();

private:
    // Translates an "A1" style address into a row/column;
    // returns true if cell currently exists:
    bool Spreadsheet::translateAddress(std::string const &address,
                                       int &rowOut, int &colOut) const;

    void Spreadsheet::copyFrom(Spreadsheet const &other);


    typedef std::vector<Cell *> CellRow;
    typedef std::vector<CellRow> Matrix;
    Matrix matrix;

    const Cell theNullCell; // A placeholder null cell 
};

#endif
